@@ -1,6 +1,6 @@
 #' Example JSON for driving
 #'
-#' Example JSON responce from OTP
+#' Example JSON response from OTP
 #' This is used for internal testing and has no use
 #'
 #' @format json
@@ -8,9 +8,16 @@
 
 #' Example JSON for transit
 #'
-#' Example JSON responce from OTP
+#' Example JSON response from OTP
 #' This is used for internal testing and has no use
 #'
 #' @format json
 "json_example_transit"
 
+#' Example JSON for driving long distance
+#'
+#' Example JSON response from OTP
+#' This is used for internal testing and has no use
+#'
+#' @format json
+"json_example_long_drive"
